@@ -5,6 +5,8 @@
 #undef DATADIR
 #undef AUTOPILOT
 #undef NETWORKING
+#undef HAVE_GTK
+#undef HAVE_QT
 
 @BOTTOM@
 #endif
