@@ -116,8 +116,10 @@ void Xarchon_State::UpdateTurn(void)
 	     (lumi == LUMI_DARKEST && !(actor->type & ACTOR_LIGHT))))
 	  cell->flags ^= CELL_IMPRISON;
 	/* heal actors, twice as fast if on a power point */
-	actor->strength = min(orig_actors_list[actor->type & ACTOR_MASK].strength,
-			      actor->strength + 2 * (1 + ((cell->flags & CELL_POWER) == CELL_POWER)));
+	actor->strength = vmin(
+            (int) orig_actors_list[actor->type & ACTOR_MASK].strength,
+            actor->strength +
+            (((cell->flags & CELL_POWER) == CELL_POWER) ? 2:1));
 	cell->flags=(cell->flags & ~CELL_LUMI_MASK) | lumi;
       }
     }
@@ -417,7 +419,7 @@ double Xarchon_Goal_Test::Goal_Value(Xarchon_State *a)
 	num_power++;
       if (a->board_cells[i][j].actor!=NULL) {
 	ACTOR *actor=cell->actor;
-	int life=field_initial_life(actor,cell); // How many strength does the actor actually has
+	int life=field_initial_life(actor,cell,NULL); // How much strength does the actor actually have
 	if (actor_is_side(actor,0)) {
 	  if (cell->flags & CELL_POWER) 
 	    light_power++;
