@@ -8,6 +8,7 @@
 #include <X11/Xlib.h>
 #include <X11/keysymdef.h>
 #include <X11/xpm.h>
+#include <X11/extensions/shape.h>
 
 #include "canvas.h"
 
@@ -33,6 +34,7 @@ static void canvas_mouse_button_event(XButtonEvent *event);
 static Display *display;
 static Window window;
 static Screen *screen;
+static int screen_num;
 static Visual *visual;
 static GC gc, gc1 = None;
 static char *keys_down = NULL;
@@ -46,6 +48,8 @@ static int canvas_width, canvas_height;
 static XRectangle *clip_rects;
 static int num_clip_rects;
 
+static Pixmap collision_map = None;
+
 static int enabled = 1;
 
 static char warp_mouse = 0;
@@ -64,6 +68,7 @@ void canvas_init(Display *_display, Window _window, int width, int height)
    window = _window;
 
    screen = DefaultScreenOfDisplay(display);
+   screen_num = DefaultScreen(display);
    visual = DefaultVisualOfScreen(screen);
    canvas_colormap = XCreateColormap(display, window, visual, AllocNone);
    canvas_black_pixel = canvas_alloc_color(0, 0, 0);
@@ -349,8 +354,9 @@ void canvas_image_load(char *filename, IMAGE *image)
    xpmattrs.valuemask = XpmSize | XpmCloseness | XpmColormap | XpmReturnPixels;
    xpmattrs.closeness = 65535;
    xpmattrs.colormap = canvas_colormap;
-   err = XpmReadFileToPixmap(display, canvas, filename,
-                             &image->pixmap, &image->mask, &xpmattrs);
+
+   err = XpmReadFileToPixmap(display, canvas, filename, 
+                           &image->pixmap, &image->mask, &xpmattrs);
    if (err != XpmSuccess) {
       fprintf(stderr, "error reading image `%s', xpm error code = %d\n", filename, err);
       exit(EXIT_FAILURE);
@@ -366,40 +372,42 @@ void canvas_image_load(char *filename, IMAGE *image)
 /* canvas_image_copy                                                        */
 /*--------------------------------------------------------------------------*/
 
-void canvas_image_copy(IMAGE *old, IMAGE *new)
+void canvas_image_copy(IMAGE *oldimg, IMAGE *newimg)
 {
    XColor *xcs;
    int i;
 
    XSetClipMask(display, gc, None);
-   new->width = old->width;
-   new->height = old->width;
-   new->pixmap = XCreatePixmap(display, canvas, new->width, new->height,
-                               DefaultDepthOfScreen(screen));
-   XCopyArea(display, old->pixmap, new->pixmap, gc,
-             0, 0, new->width, new->height, 0, 0);
+   newimg->width = oldimg->width;
+   newimg->height = oldimg->width;
+   newimg->pixmap = XCreatePixmap(display, canvas,
+                                  newimg->width, newimg->height,
+                                  DefaultDepthOfScreen(screen));
+   XCopyArea(display, oldimg->pixmap, newimg->pixmap, gc,
+             0, 0, newimg->width, newimg->height, 0, 0);
 
-   new->colors = malloc(sizeof(long) * old->num_colors);
-   memcpy(new->colors, old->colors, sizeof(long) * old->num_colors);
-   new->num_colors = old->num_colors;
-   xcs = malloc(sizeof(XColor) * new->num_colors);
-   for (i = 0; i < new->num_colors; i++)
-      xcs[i].pixel = new->colors[i];
-   XQueryColors(display, canvas_colormap, xcs, new->num_colors);
-   for (i = 0; i < new->num_colors; i++) {
+   newimg->colors = malloc(sizeof(long) * oldimg->num_colors);
+   memcpy(newimg->colors, oldimg->colors, sizeof(long) * oldimg->num_colors);
+   newimg->num_colors = oldimg->num_colors;
+   xcs = malloc(sizeof(XColor) * newimg->num_colors);
+   for (i = 0; i < newimg->num_colors; i++)
+      xcs[i].pixel = newimg->colors[i];
+   XQueryColors(display, canvas_colormap, xcs, newimg->num_colors);
+   for (i = 0; i < newimg->num_colors; i++) {
       xcs[i].flags = DoRed | DoGreen | DoBlue;
       XAllocColor(display, canvas_colormap, &xcs[i]);
    }
    free(xcs);
 
-   if (old->mask != None) {
-      new->mask = XCreatePixmap(display, canvas, new->width, new->height, 1);
+   if (oldimg->mask != None) {
+      newimg->mask = XCreatePixmap(display, canvas,
+                                   newimg->width, newimg->height, 1);
       if (gc1 == None)
-         gc1 = XCreateGC(display, old->mask, 0, NULL);
-      XCopyArea(display, old->mask, new->mask, gc1,
-                0, 0, new->width, new->height, 0, 0);
+         gc1 = XCreateGC(display, oldimg->mask, 0, NULL);
+      XCopyArea(display, oldimg->mask, newimg->mask, gc1,
+                0, 0, newimg->width, newimg->height, 0, 0);
    } else
-      new->mask = None;
+      newimg->mask = None;
 }
 
 /*--------------------------------------------------------------------------*/
@@ -544,20 +552,85 @@ void canvas_image_free(IMAGE *image)
 /* canvas_image_paint                                                       */
 /*--------------------------------------------------------------------------*/
 
-void canvas_image_paint(IMAGE *image, int x, int y, int sx, int sy, int sw, int sh)
+void canvas_image_paint(IMAGE *image, IMAGE *mask_image, int x, int y, int sx, int sy, int sw, int sh)
 {
    XGCValues gcv;
 
    if (!enabled)
       return;
-   gcv.clip_x_origin = x;
-   gcv.clip_y_origin = y;
-   gcv.clip_mask = image->mask;
+   gcv.clip_x_origin = x - sx;
+   gcv.clip_y_origin = y - sy;
+   gcv.clip_mask = mask_image->mask;
    XChangeGC(display, gc, GCClipXOrigin | GCClipYOrigin | GCClipMask, &gcv);
    XCopyArea(display, image->pixmap, canvas, gc, sx, sy, sw, sh, x, y);
    canvas_dirty(x, y, image->width, image->height);
 }
 
+/*--------------------------------------------------------------------------*/
+/* canvas_image_intersect                                                   */
+/*--------------------------------------------------------------------------*/
+
+int canvas_image_intersect(IMAGE *image1, IMAGE *image2, int x_off, int y_off)
+{
+   XImage *intersection;
+   int y1, y2;
+   int sub_w, sub_h;
+   int x, y;
+   int result = 0;
+
+   if (image1->mask == None && image2->mask == None)
+      return 1;         /* if they're both rectangular, and we've got here, */
+                        /* then they intersect by definition */
+   if (gc1 == None) {
+      if (image1->mask == None)
+         gc1 = XCreateGC(display, image2->mask, 0, NULL);
+      else
+         gc1 = XCreateGC(display, image1->mask, 0, NULL);
+   }
+   XSetForeground(display, gc1, BlackPixel(display, screen_num));
+   if (collision_map == None) {
+      collision_map = XCreatePixmap(display, window,
+            image1->width, image1->height, 1);
+      XFillRectangle(display, collision_map, gc1, 0, 0, image1->width,
+            image1->height);
+   }
+   sub_w = image1->width - x_off;
+   if (y_off < 0) {
+      y1 = -y_off;
+      y2 = 0;
+      sub_h = image1->height + y_off;
+   } else {
+      y1 = 0;
+      y2 = y_off;
+      sub_h = image1->height - y_off;
+   }
+   XSetClipOrigin(display, gc1, 0, 0);
+   /* if image1->mask == None, this is fine - turning off the clip mask is */
+   /* the right thing to do in that circumstance (image1 being rectangular) */
+   XSetClipMask(display, gc1, image1->mask);
+   if (image2->mask != None)
+      XCopyArea(display, image2->mask, collision_map, gc1, 0, y1, sub_w,
+            sub_h, x_off, y2);
+   else {
+      /* if image2->mask == None, use the whole of images2's rectangle */
+      /* (clipped with image1's mask) */
+      XSetForeground(display, gc1, WhitePixel(display, screen_num));
+      XFillRectangle(display, collision_map, gc1, x_off, y2, sub_w, sub_h);
+      XSetForeground(display, gc1, BlackPixel(display, screen_num));
+   }
+   XSetClipMask(display, gc1, None);
+   intersection = XGetImage(display, collision_map, x_off, y2, sub_w, sub_h,
+         AllPlanes, ZPixmap);
+   XFillRectangle(display, collision_map, gc1, x_off, y2, sub_w, sub_h);
+   for (y = 0; !result && y < intersection->height; y++)
+      for (x = 0; !result && x < intersection->width; x++) {
+         if (XGetPixel(intersection, x, y))
+            result = 1;
+      }
+   XDestroyImage(intersection);
+   return result;
+}
+
 /*--------------------------------------------------------------------------*/
 /* canvas_key_event                                                         */
 /*--------------------------------------------------------------------------*/
@@ -640,10 +713,10 @@ void canvas_mouse_init(int absolute)
 }
 
 /*--------------------------------------------------------------------------*/
-/* canvas_mouse_cursor                                                      */
+/* canvas_init_cursor                                                       */
 /*--------------------------------------------------------------------------*/
 
-void canvas_mouse_cursor(int on)
+void canvas_init_cursor(int on)
 {
    XColor c;
    Pixmap p;
@@ -718,9 +791,9 @@ void canvas_mouse_button_event(XButtonEvent *event)
 void canvas_mouse_event(XEvent *event)
 {
   if (event->type == MotionNotify)
-     canvas_mouse_drag_event(event);
+     canvas_mouse_drag_event((XMotionEvent *)event);
   if (event->type == ButtonPress || event->type == ButtonRelease)
-     canvas_mouse_button_event(event);
+     canvas_mouse_button_event((XButtonEvent *)event);
 }
 
 /*--------------------------------------------------------------------------*/
