@@ -5,7 +5,6 @@
 #include <config.h>
 
 #include <limits.h>
-#include <signal.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -16,12 +15,6 @@
 #include <X11/Xlib.h>
 #include <X11/xpm.h>
 
-#include <gdk/gdkx.h>
-#include <gtk/gtk.h>
-#include "gtk-callbacks.h"
-#include "gtk-interface.h"
-#include "gtk-support.h"
-
 #include "main.h"
 #include "canvas.h"
 #include "theme.h"
@@ -29,6 +22,7 @@
 #include "iface.h"
 #include "audio.h"
 #include "field.h"
+#include "toolkit.h"
 
 #include "main-random.h"
 
@@ -39,20 +33,17 @@
 #define APP_NAME        "xarchon"
 
 #define UPF             (1000000 / FPS)         /* usecs per frame */
-#define TIMER_RES       10000                   /* OS timer resolution */
-#define UPF_MIN         (UPF - UPF % TIMER_RES)
-#define UPF_MAX         (UPF + UPF % TIMER_RES)
+
+#define CONFIG_VERSION  5
 
 /*--------------------------------------------------------------------------*/
 /* functions                                                                */
 /*--------------------------------------------------------------------------*/
 
-static void set_sensitive_options(int game_active);
-static void set_visible_gui(int visible);
 static void progress_func(char *msg, float progress);
-static void init_gui(int *argc, char *argv[]);
-static void init(int *argc, char *argv[]);
-static void play(int n);
+static void init_gui(void);
+static void init(int argc, char **argv);
+static void play(void);
 static void loop(void);
 
 /*--------------------------------------------------------------------------*/
@@ -61,13 +52,13 @@ static void loop(void);
 
 int random_index;
 
-static GtkWidget *window, *menubar, *progressbar;
-
 static Atom wm_protocols, wm_delete_window;
 
 static int initializing = 1;
 static int quitting = 0;
 
+static char *private_config, *public_config;
+
 /*--------------------------------------------------------------------------*/
 /* main_random                                                              */
 /*--------------------------------------------------------------------------*/
@@ -82,103 +73,68 @@ int main_random(void)
 }
 
 /*--------------------------------------------------------------------------*/
-/* main_msgbox                                                              */
+/* main_expose_event                                                        */
 /*--------------------------------------------------------------------------*/
 
-void main_msgbox(char *msg)
+void main_expose_event(void)
 {
-   GtkWidget *dialog, *label, *okay_button;
-
-   dialog = gtk_dialog_new();
-   label = gtk_label_new(msg);
-   okay_button = gtk_button_new_with_label("OK");
-   gtk_signal_connect_object(GTK_OBJECT(okay_button), "clicked",
-                             GTK_SIGNAL_FUNC(gtk_widget_destroy),
-                             GTK_OBJECT(dialog));
-   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->action_area),
-                     okay_button);
-   gtk_container_add(GTK_CONTAINER(GTK_DIALOG(dialog)->vbox), label);
-   gtk_window_set_modal(GTK_WINDOW(dialog), 1);
-   gtk_widget_show_all(dialog);
+   /* redraw canvas only if a game is in progress */
+   if (!toolkit_is_toolkit_active() && board_pause_game(-1))
+      canvas_refresh_all();
 }
 
 /*--------------------------------------------------------------------------*/
-/* set_sensitive_options                                                    */
+/* main_destroy_event                                                       */
 /*--------------------------------------------------------------------------*/
 
-static void set_sensitive_options(int game_active)
+void main_destroy_event(void)
 {
-   gtk_widget_set_sensitive(lookup_widget(window, "mnu_game_new"),
-                            !game_active);
-   gtk_widget_set_sensitive(lookup_widget(window, "mnu_game_stop"),
-                            game_active);
-   gtk_widget_set_sensitive(lookup_widget(window, "mnu_settings_define"),
-                            !game_active);
-   gtk_widget_set_sensitive(lookup_widget(window, "mnu_settings_select"),
-                            !game_active);
-   gtk_widget_set_sensitive(lookup_widget(window, "mnu_settings_theme"),
-                            !game_active);
+   if (initializing)
+      exit(EXIT_SUCCESS);
+   quitting = 1;
+   toolkit_set_toolkit_active(0);
 }
 
 /*--------------------------------------------------------------------------*/
-/* set_visible_gui                                                          */
+/* main_new_game                                                            */
 /*--------------------------------------------------------------------------*/
 
-static void set_visible_gui(int visible)
-{
-   if (visible) {
-      gtk_widget_show(menubar);
-      gtk_widget_show(lookup_widget(window, "logo"));
-      gtk_widget_show(progressbar);
-   } else {
-      gtk_widget_hide(menubar);
-      gtk_widget_hide(lookup_widget(window, "logo"));
-      gtk_widget_hide(progressbar);
-   }
-   gdk_flush();
-}
-
-/*--------------------------------------------------------------------------*/
-/* main_game_new                                                            */
-/*--------------------------------------------------------------------------*/
-
-void main_game_new(GtkMenuItem *menuitem, gpointer user_data)
+void main_new_game(int game_type)
 {
    char *msg;
-   int light_first;
+   int light_or_order_first;
 
-   set_visible_gui(0);
-   msg = iface_start(&light_first);
+   toolkit_set_visible_gui(0);
+   msg = iface_start(&light_or_order_first);
    quitting = 0;                        /* this is important */
    if (msg != NULL) {
-      set_visible_gui(1);
+      toolkit_set_visible_gui(1);
       progress_func(msg, 1.0);
       return;
    }
-   set_sensitive_options(1);
-   board_start_game(light_first);
-   if (menuitem != NULL)                /* NULL when called by main() */
-      gtk_main_quit();
+   toolkit_set_sensitive_options(1);
+   board_start_game(game_type, light_or_order_first);
+   toolkit_set_toolkit_active(0);
 }
 
 /*--------------------------------------------------------------------------*/
 /* main_game_unpause                                                        */
 /*--------------------------------------------------------------------------*/
 
-void main_game_unpause(GtkMenuItem *menuitem, gpointer user_data)
+void main_game_unpause(void)
 {
    if (board_pause_game(0))
-      gtk_main_quit();
+      toolkit_set_toolkit_active(0);
 }
 
 /*--------------------------------------------------------------------------*/
 /* main_game_stop                                                           */
 /*--------------------------------------------------------------------------*/
 
-void main_game_stop(GtkMenuItem *menuitem, gpointer user_data)
+void main_game_stop(void)
 {
    board_end_game();
-   set_sensitive_options(0);
+   toolkit_set_sensitive_options(0);
    progress_func("Game Ended", 1.0);
 }
 
@@ -186,7 +142,7 @@ void main_game_stop(GtkMenuItem *menuitem, gpointer user_data)
 /* main_toggle_sound                                                        */
 /*--------------------------------------------------------------------------*/
 
-void main_toggle_sound(GtkMenuItem *menuitem, gpointer user_data)
+void main_toggle_sound(void)
 {
    if (audio_isenabled())
       audio_disable();
@@ -198,74 +154,10 @@ void main_toggle_sound(GtkMenuItem *menuitem, gpointer user_data)
 /* main_game_exit                                                           */
 /*--------------------------------------------------------------------------*/
 
-void main_game_exit(GtkMenuItem *menuitem, gpointer user_data)
+void main_game_exit(void)
 {
    quitting = 1;
-   gtk_main_quit();
-}
-
-/*--------------------------------------------------------------------------*/
-/* main_help_tips                                                           */
-/*--------------------------------------------------------------------------*/
-
-void main_help_tips(GtkMenuItem *menuitem, gpointer user_data)
-{
-   static GtkWidget *window = NULL;
-   static int destroying = 0;
-
-   if (window == NULL) {
-      window = create_tips_window();
-      gtk_widget_show(window);
-   } else if (!destroying) {
-      destroying = 1;
-      gtk_widget_destroy(window);
-      window = NULL;
-      destroying = 0;
-   }
-}
-
-/*--------------------------------------------------------------------------*/
-/* main_help_about                                                          */
-/*--------------------------------------------------------------------------*/
-
-void main_help_about(GtkMenuItem *menuitem, gpointer user_data)
-{
-   static GtkWidget *window = NULL;
-   static int destroying = 0;
-
-   if (window == NULL) {
-      window = create_about_window();
-      gtk_widget_show(window);
-   } else if (!destroying) {
-      destroying = 1;
-      gtk_widget_destroy(window);
-      window = NULL;
-      destroying = 0;
-   }
-}
-
-/*--------------------------------------------------------------------------*/
-/* main_window_expose_event                                                 */
-/*--------------------------------------------------------------------------*/
-
-gboolean main_window_expose_event(GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
-{
-   /* redraw canvas only if a game is in progress */
-   if (gtk_main_level() == 0 && board_pause_game(-1))
-      canvas_refresh_all();
-   return TRUE;
-}
-
-/*--------------------------------------------------------------------------*/
-/* main_window_destroy_event                                                */
-/*--------------------------------------------------------------------------*/
-
-void main_window_destroy_event(GtkObject *object, gpointer data)
-{
-   if (initializing)
-      exit(EXIT_SUCCESS);
-   quitting = 1;
-   gtk_main_quit();
+   toolkit_set_toolkit_active(0);
 }
 
 /*--------------------------------------------------------------------------*/
@@ -274,65 +166,125 @@ void main_window_destroy_event(GtkObject *object, gpointer data)
 
 void progress_func(char *msg, float progress)
 {
-   if (progress == 0.0)
-      gtk_widget_set_sensitive(menubar, FALSE);
-   if (progress == 1.0)
-      gtk_widget_set_sensitive(menubar, TRUE);
-   gtk_progress_set_format_string(GTK_PROGRESS(progressbar), msg);
-   gtk_progress_bar_update(GTK_PROGRESS_BAR(progressbar), progress);
-   if (gtk_main_level() == 0) {
-      while (gtk_events_pending())
-         gtk_main_iteration();
-      gdk_flush();
+   toolkit_set_progressbar(msg, progress);
+}
+
+/*--------------------------------------------------------------------------*/
+/* main_config_read                                                         */
+/*--------------------------------------------------------------------------*/
+
+void main_config_read(void)
+{
+   FILE *fp;
+   int version;
+
+   if (private_config == NULL) {
+      if (!getenv("HOME")) {
+         fprintf(stderr, "Your home directory is not known.\n"
+                         "Please set the HOME environment variable to your home directory.\n");
+         exit(EXIT_FAILURE);
+      }
+      private_config = malloc(PATH_MAX);
+      sprintf(private_config, "%s/.xarchon", getenv("HOME"));
    }
+   fp = fopen(private_config, "r");
+
+   if (fp != NULL) {
+      fscanf(fp, "%d", &version);
+      if (version < CONFIG_VERSION) {
+         fprintf(stderr, "Your configuration file was created by an older version of this program.\n"
+                         "Its format is no longer readable by this version.  It will be deleted.\n");
+         fclose(fp);
+         unlink(private_config);
+         fp = NULL;
+      }
+   }
+   
+   if (fp == NULL) {
+      if (public_config == NULL) {
+         public_config = malloc(PATH_MAX);
+         sprintf(public_config, "%s/xarchon.default", DATADIR);
+      }
+      fp = fopen(public_config, "r");
+      if (fp == NULL) {
+         fprintf(stderr, "Could not find either `%s' or `%s'.  Giving up.\n",
+                 private_config, public_config);
+         exit(EXIT_FAILURE);
+      }
+
+      fscanf(fp, "%d", &version);
+      if (version != CONFIG_VERSION) {
+         fprintf(stderr, "Unknown version code in configuration file.  Giving up.\n");
+         exit(EXIT_FAILURE);
+      }
+   }
+
+   toolkit_config_read(fp);
+   iface_config_read(fp);
+
+   fclose(fp);
+}
+
+/*--------------------------------------------------------------------------*/
+/* main_config_write                                                        */
+/*--------------------------------------------------------------------------*/
+
+void main_config_write(void)
+{
+   FILE *fp;
+   
+   fp = fopen(private_config, "w");
+   if (fp == NULL) {
+      fprintf(stderr, "Could not create `%s'.\n", private_config);
+      return;
+   }
+   
+   fprintf(fp, "%d\n", CONFIG_VERSION);
+   
+   toolkit_config_write(fp);
+   iface_config_write(fp);
+   
+   fclose(fp);
 }
 
 /*--------------------------------------------------------------------------*/
 /* init_gui                                                                 */
 /*--------------------------------------------------------------------------*/
 
-void init_gui(int *argc, char *argv[])
+void init_gui(void)
 {
-   GtkPixmap *pixmap;
+   Display *display;
+   Window window;
 
-   gtk_init(argc, &argv);
-   wm_protocols = XInternAtom(GDK_DISPLAY(), "WM_PROTOCOLS", False);
-   wm_delete_window = XInternAtom(GDK_DISPLAY(), "WM_DELETE_WINDOW", False);
-   add_pixmap_directory(DATADIR);       /* a glade thingy */
+   toolkit_create_main_window();
 
-   window = create_main_window();
-   gtk_widget_set_usize(window, CANVAS_WIDTH, CANVAS_HEIGHT);
+   display = toolkit_get_xdisplay();
+   window = toolkit_get_xwindow();
+   
+   wm_protocols = XInternAtom(display, "WM_PROTOCOLS", False);
+   wm_delete_window = XInternAtom(display, "WM_DELETE_WINDOW", False);
 
-   gtk_signal_connect(GTK_OBJECT(window), "delete_event",
-                     GTK_SIGNAL_FUNC(gtk_false), NULL);
-
-   menubar = lookup_widget(window, "menubar");
-   progressbar = lookup_widget(window, "progressbar");
-   set_sensitive_options(0);
-   gtk_widget_set_sensitive(menubar, FALSE);
-
-   gtk_widget_show(window);
-   XSelectInput(GDK_DISPLAY(), GDK_WINDOW_XWINDOW(window->window),
+   XSelectInput(display, window,
                 KeyPressMask | KeyReleaseMask |
                 PointerMotionMask | ButtonPressMask | ButtonReleaseMask |
                 ExposureMask | FocusChangeMask);
-
-   pixmap = GTK_PIXMAP(create_pixmap(window, "icon.xpm"));
-   gdk_window_set_icon(window->window, NULL, pixmap->pixmap, pixmap->mask);
-   gtk_widget_destroy(GTK_WIDGET(pixmap));
 }
 
 /*--------------------------------------------------------------------------*/
 /* init                                                                     */
 /*--------------------------------------------------------------------------*/
 
-void init(int *argc, char *argv[])
+void init(int argc, char *argv[])
 {
    char str[64];
+   Display *dpy;  Window w;
 
-   iface_config_read();
-   init_gui(argc, argv);
-   canvas_init(GDK_DISPLAY(), GDK_WINDOW_XWINDOW(window->window),
+   toolkit_init(argc, argv);
+   main_config_read();
+   init_gui();
+   dpy = toolkit_get_xdisplay();
+   w = toolkit_get_xwindow();
+   canvas_init(dpy, w,
                CANVAS_WIDTH, CANVAS_HEIGHT);
    canvas_install_colormap(0);
    audio_init();
@@ -398,11 +350,13 @@ static void display_event(XEvent *ev)
 
 int main_handle_events(int in_board_game)
 {
+   Display *display;
    XEvent ev;
    int key;
 
-   while (XEventsQueued(GDK_DISPLAY(), QueuedAfterReading) > 0) {
-      XNextEvent(GDK_DISPLAY(), &ev);
+   display = toolkit_get_xdisplay();
+   while (XEventsQueued(display, QueuedAfterReading) > 0) {
+      XNextEvent(display, &ev);
 #ifdef DEBUG
       display_event(&ev);
 #endif
@@ -413,7 +367,7 @@ int main_handle_events(int in_board_game)
                /* if Escape on game that has ended, or if F12 */
                if ((key == XK_Escape && !board_pause_game(-1)) ||
                    key == XK_F12) {
-                  main_game_stop(NULL, NULL);
+                  main_game_stop();
                   return 0;
                }
                /* if Escape on a non-network game */
@@ -424,7 +378,7 @@ int main_handle_events(int in_board_game)
                }
                /* if F11 */
                if (key == XK_F11)
-                  main_toggle_sound(NULL, NULL);
+                  main_toggle_sound();
             }
             /* fallthrough */
          case KeyRelease:
@@ -453,6 +407,7 @@ int main_handle_events(int in_board_game)
                quitting = 1;
                return 0;
             }
+            break;
       }
    }
    return 1;
@@ -462,30 +417,31 @@ int main_handle_events(int in_board_game)
 /* play                                                                     */
 /*--------------------------------------------------------------------------*/
 
-void play(int n)
+void play()
 {
-   struct itimerval itv;
+   int delay;
+   struct timeval before, after;
 
-   signal(SIGALRM, play);               /* always re-install signal handler */
-   if (n != 0)                          /* if acting as signal handler */
-      return;
-   itv.it_interval.tv_sec = itv.it_value.tv_sec = 0;
+   delay = UPF;
+   gettimeofday(&before, NULL);
    while (1) {
-      /* timing logic:  at 40 fps, we need to get control every 25000us, */
-      /* but the Linux timer ticks at 10ms == 10000us.  Control would be */
-      /* returned only after 30000us, which is like working at 33 fps.   */
-      /* Workaround:  alternate the pause between 20000 and 30000 usec,  */
-      /* keeping an average of 25000us per frame, which is 40 fps.       */
-      if (itv.it_interval.tv_usec == UPF_MIN)
-         itv.it_interval.tv_usec = itv.it_value.tv_usec = UPF_MAX;
-      else
-         itv.it_interval.tv_usec = itv.it_value.tv_usec = UPF_MIN;
-      setitimer(ITIMER_REAL, &itv, NULL);
-      pause();
-      signal(SIGALRM, play);            /* re-install signal handler */
       if (!main_handle_events(1))
          return;
       board_frame();
+      /* timing logic:  at 40 fps, we need to get control every 25000us, */
+      /* but the Linux timer ticks at 10ms == 10000us.  Control would be */
+      /* returned only after 30000us, which is like working at 33 fps.   */
+      /* Workaround:  track the actual time taken, and correct the pause */
+      /* to average to the desired 25000us per frame.  */
+      usleep(delay);
+      gettimeofday(&after, NULL);
+      /* correct the interval by the number of usecs over UPF we went */
+      delay -= 1000000*(after.tv_sec - before.tv_sec) +
+            after.tv_usec - before.tv_usec - UPF;
+      if (delay <= 0)
+         /* we've accumulated so much error that we've lost a frame */
+         delay = UPF;
+      before = after;
    }
 }
 
@@ -496,19 +452,19 @@ void play(int n)
 void loop(void)
 {
    while (1) {
-      gtk_main();
+      toolkit_set_toolkit_active(1);
       if (quitting)
          return;
-      set_visible_gui(0);
+      toolkit_set_visible_gui(0);
       canvas_install_colormap(1);
       canvas_refresh_all();
-      canvas_mouse_cursor(1);
-      play(0);
-      canvas_mouse_cursor(0);
+      canvas_init_cursor(1);
+      play();
+      canvas_init_cursor(0);
       if (quitting)
          return;
       canvas_install_colormap(0);
-      set_visible_gui(1);
+      toolkit_set_visible_gui(1);
    }
 }
 
@@ -520,7 +476,7 @@ void loop(void)
 
 int main(int argc, char *argv[])
 {
-   init(&argc, argv);
+   init(argc, argv);
 
 #ifndef AUTOPILOT
    loop();
@@ -543,7 +499,7 @@ int main(int argc, char *argv[])
    for (i = 0; i < 20; i++)
    for (lgt = ACTOR_LIGHT_FIRST; lgt <= ACTOR_LIGHT_LAST; lgt++)
       for (drk = ACTOR_DARK_FIRST; drk <= ACTOR_DARK_LAST; drk++) {
-         board_start_game(lgt);
+         board_start_game(GAME_ARCHON, lgt);
          printf("* * * * * actor %d battles %d * * * * * (%d)\n", lgt, drk, i);
          fprintf(stderr, "* * * * * actor %d battles %d * * * * * (%d)\n", lgt, drk, i);
          board_find_actor(lgt, &lgt_x, &lgt_y);
