@@ -21,11 +21,11 @@
 #include <unistd.h>
 #endif
 
+#include "main.h"
 #include "network.h"
 #include "actors.h"
 #include "canvas.h"
 #include "board.h"
-#include "main.h"
 
 /*--------------------------------------------------------------------------*/
 /* define                                                                   */
@@ -526,68 +526,3 @@ void network_config_write(FILE *fp, NETWORK_CONFIG *config)
 {
    fprintf(fp, "%-64s %d\n", config->address, config->port);
 }
-
-/****************************************************************************/
-/*                                                                          */
-/* GTK+ Stuff                                                               */
-/*                                                                          */
-/****************************************************************************/
-
-#include <gtk/gtk.h>
-#include "gtk-callbacks.h"
-#include "gtk-interface.h"
-#include "gtk-support.h"
-
-/*--------------------------------------------------------------------------*/
-/* variables                                                                */
-/*--------------------------------------------------------------------------*/
-
-static NETWORK_CONFIG *orig_config;
-static GtkWidget *window;
-
-/*--------------------------------------------------------------------------*/
-/* network_config_edit                                                      */
-/*--------------------------------------------------------------------------*/
-
-void network_config_edit(NETWORK_CONFIG *_config)
-{
-   GtkWidget *widget;
-   char port[5];
-
-   orig_config = _config;
-   window = create_network_window();
-   widget = lookup_widget(window, "address");
-   gtk_entry_set_text(GTK_ENTRY(widget), orig_config->address);
-   widget = lookup_widget(window, "port");
-   sprintf(port, "%d", orig_config->port);
-   gtk_entry_set_text(GTK_ENTRY(widget), port);
-   gtk_widget_show(window);
-}
-
-/*--------------------------------------------------------------------------*/
-/* network_ok_clicked                                                       */
-/*--------------------------------------------------------------------------*/
-
-void network_ok_clicked(GtkButton *button, gpointer data)
-{
-   GtkWidget *widget;
-   char *str;
-   char port[8];
-
-   widget = lookup_widget(window, "address");
-   str = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, sizeof(orig_config->address) - 2);
-   strcpy(orig_config->address, str);
-   widget = lookup_widget(window, "port");
-   str = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, sizeof(port) - 2);
-   orig_config->port = atoi(str);
-   gtk_widget_destroy(window);
-}
-
-/*--------------------------------------------------------------------------*/
-/* network_cancel_clicked                                                   */
-/*--------------------------------------------------------------------------*/
-
-void network_cancel_clicked(GtkObject *object, gpointer data)
-{
-   gtk_widget_destroy(window);
-}
